 --
-- EngToJpn+.lua
-- Convert ENG Phonemes to JPN.

--
-- Copyright (C) pastro99 (Based on Voctro Labs file architecture)
--

--
-- Manifest
--
function manifest()
    myManifest = {
        name          = "EngToJpn+",
        comment       = "Convert English Phonemes to Japanese",
        author        = "pastro99",
        pluginID      = "{F17A37C4-AD2C-433a-B5BC-773F2EACB94C}",
        pluginVersion = "2.5.0.0",
        apiVersion    = "3.0.0.1"
    }
    
    return myManifest
end



--
-- Main
--
function main(processParam, envParam)
	-- -- Process parameters
	-- local beginPosTick = processParam.beginPosTick	-- begin of time range to apply Job to (in ticks)
	-- local endPosTick   = processParam.endPosTick		-- end of time range to apply Job to (in ticks)
	-- local songPosTick  = processParam.songPosTick	-- onset of song (in ticks)

	-- -- Environment parameters
	-- local scriptDir  = envParam.scriptDir	-- script folder
	-- local scriptName = envParam.scriptName	-- script name
	-- local tempDir    = envParam.tempDir		-- temporary folder
	
	local mapping    = {}
	local mapping2   = {}
	local noteEx     = {}
	local noteExList = {}
	local noteCount
	local retCode
	local idx
	
	-- define English to Japanese phoneme mapping
	mapping["@"]	= "a"
				mapping2["@ z"]  = {"a", "z"} 
	mapping["V"]	= "a"
				mapping2["V v"]  = {"o", "p\\"} 
				mapping2["V z"]  = {"a", "z"} 
	mapping["e"]	= "e"
				mapping2["e z"]  = {"e", "z"} 
	mapping["I"]	= "i"
				mapping2["I z"]  = {"i", "z"} 
	mapping["i:"]	= "i i"
				mapping2["i: z"] = {"i i", "z"} 
	mapping["{"]	= "a"
				mapping2["{ z"]  = {"a", "z"} 
	mapping["O:"]	= "o"
				mapping2["O: z"] = {"o", "z"} 
	mapping["Q"]	= "o"
				mapping2["Q z"]  = {"o", "z"} 
	mapping["U"]	= "M"
				mapping2["U z"]  = {"M", "z"} 
	mapping["u:"]	= "M"
				mapping2["u: z"] = {"M", "z"} 

	mapping["@r"]	= "a 4"
				mapping2["@r l"] = {"a", "4"} 
	mapping["I@"]	= "i a 4"
	mapping["e@"]	= "e a 4"
	mapping["U@"]	= "u a 4"
	mapping["O@"]	= "o a 4"
	mapping["Q@"]	= "a 4"

	mapping["eI"]	= "e i"
	mapping["aI"]	= "a i"
	mapping["OI"]	= "o i"

	mapping["@U"]	= "o w"
	mapping["aU"]	= "a w"
	
	mapping["m"]	= "m"
				mapping2["m I"] = {"m'", "i"} 
				mapping2["m i:"]= {"m'", "i"} 
				mapping2["m I@"]= {"m'", "a 4"}
	mapping["n"]	= "n"
				mapping2["n I"] = {"J", "i"} 
				mapping2["n i:"]= {"J", "i"}
				mapping2["n I@"]= {"J", "a 4"} 
	mapping["N"]	= "N"

	mapping["w"]	= "w"
	
	mapping["j"]	= "j"

	mapping["r"]	= "4"
				mapping2["r I"] = {"4'", "i"} 
				mapping2["r i:"]= {"4'", "i"} 
				mapping2["r I@"]= {"4'", "a 4"}
	mapping["l"]	= "4"
				mapping2["l V"]= {"4", "a"} 
	mapping["l0"]	= "4"
				mapping2["l0 I"] = {"4'", "i"} 
				mapping2["l0 i:"]= {"4'", "i"} 
				mapping2["l0 V"] = {"4", "a"} 
				mapping2["10 I@"]= {"4'", "a 4"}
	mapping["p"]	= "p"
	mapping["t"]	= "t"
				mapping2["t s"]= {"ts", ""}
	mapping["k"]	= "k"

	mapping["ph"]	= "p"
				mapping2["ph I"] = {"p'", "i"} 
				mapping2["ph i:"]= {"p'", "i"} 
				mapping2["ph j"] = {"p'", ""} 
				mapping2["ph I@"]= {"p'", "a 4"}
	mapping["gh"]	= "g"
				mapping2["gh I"] = {"g'", "i"} 
				mapping2["gh i:"]= {"g'", "i"}
				mapping2["gh j"] = {"g'",""} 
				mapping2["gh I@"]= {"g'", "a 4"}
	mapping["th"]	= "t"
				mapping2["th I"] = {"t'", "i"} 
				mapping2["th i:"]= {"t'", "i"}
				mapping2["th j"] = {"t'",""} 
				mapping2["th I@"]= {"t'", "a 4"}
	mapping["kh"]	= "k"
				mapping2["kh I"] = {"k'", "i"} 
				mapping2["kh i:"]= {"k'", "i"}
				mapping2["kh j"] = {"k'",""} 
				mapping2["kh I@"]= {"k'", "a 4"}
				mapping2["kh {"] = {"k'", "a"} 
	mapping["bh"]	= "b"
				mapping2["bh I"] = {"b'", "i"} 
				mapping2["bh i:"]= {"b'", "i"} 
				mapping2["bh j"] = {"b'",""} 
				mapping2["bh I@"]= {"b'", "a 4"}
	mapping["dh"]	= "d"
				mapping2["dh I"] = {"d'", "i"} 
				mapping2["dh i:"]= {"d'", "i"}
				mapping2["dh j"] = {"d'",""}
				mapping2["dh I@"]= {"d'", "a 4"}   
				mapping2["dh V"] = {"d", "a"} 

	mapping["b"]	= "b"
				mapping2["b V"]= {"b", "a"} 
	mapping["d"]	= "d"
	mapping["g"]	= "g"

	mapping["tS"]	= "tS"
	mapping["dZ"]	= "dZ"

	mapping["f"]	= "p\\"
				mapping2["f I"] = {"p\\'", "i"} 
				mapping2["f i:"]= {"p\\'", "i"} 
				mapping2["f I@"]= {"p\\'", "a 4"} 
	mapping["h"]	= "h"
				mapping2["h I"] = {"C", "i"} 
				mapping2["h i:"]= {"C", "i"} 
				mapping2["h I@"]= {"C", "a 4"} 
	mapping["T"]	= "S"
				mapping2["T I@"]= {"S", "a 4"} 
	mapping["s"]	= "s"
	mapping["S"]	= "S"

	mapping["v"]	= "b w"

	mapping["z"]	= "dz"
	mapping["Z"]	= "dZ"
	mapping["D"]	= "dz"

	mapping["R"]	= "4"
				mapping2["R I"] = {"4'", "i"} 
				mapping2["R i:"]= {"4'", "i"} 
				mapping2["R I@"]= {"4'", "a 4"}
	mapping["?"]	= "Sil"




	
	
	
	-- get list of notes
	VSSeekToBeginNote()
	idx = 1
	retCode, noteEx = VSGetNextNoteEx()
	while (retCode == 1) do
		noteExList[idx] = noteEx
		retCode, noteEx = VSGetNextNoteEx()
		idx = idx + 1
	end

	-- check if empty
	noteCount = table.getn(noteExList)
	if (noteCount == 0) then
		VSMessageBox("Invalid data.", 0)
		return 0
	end
	
	-- transform notes
	for idx = 1, noteCount do
		local updNoteEx = {}
		updNoteEx = noteExList[idx]
		
		-- split on whitespace
		local phns = {}
		for token in string.gmatch(updNoteEx.phonemes, "[^%s]+") do
			 table.insert(phns, token)
		end
		
		-- do phoneme-by-phoneme mapping
		for i = 1, table.getn(phns) do
			-- try exceptions first
			local mapped = false
			if i+1 <= table.getn(phns) then
				twophn = phns[i] .. " " .. phns[i+1]
				if mapping2[twophn] then
					phns[i] = mapping2[twophn][1]
					phns[i+1] = mapping2[twophn][2]
					-- (in this case no need to increment i because mapping2[twophn][2] is always "i", so will map to "i" on next iteration)
					mapped = true -- there's no continue in lua
				end
			end
			
			-- general mapping second
			if not mapped then
				if mapping[phns[i]] then
					phns[i] = mapping[phns[i]]
				end
			end
		end
		
		-- join by space
		updNoteEx.phonemes = table.concat(phns, " ");
		
		-- update note
		retCode = VSUpdateNoteEx(updNoteEx);
		if (retCode ~= 1) then
			VSMessageBox("Failed to update note!", 0)
			return 1
		end
	end


	-- Success
	return 0
end

-- Check if note has phLock property (only in newer API, Editor 3.0.5+)
function canPhLock(note)
	local status = false

	for key, val in pairs(note) do
		if (key == "phLock") then
			status = true
			break
		end
	end
	
	return status
end